 /*
 *
 * Made for Marjorie with Love by Kevin - 05/04/21
 * Components:
 * Arduino Mega: https://www.amazon.com/gp/product/B016JWNYBE
 * RAMPS Controller Board: https://www.amazon.com/gp/product/B01E5K98C6
 * Display: https://www.amazon.com/gp/product/B01FH8KTZU
 * SparkFun Environmental Combo Breakout -CCS811/BME280 Sensor: https://www.amazon.com/gp/product/B076955G5S
 * Qwiic cables: https://www.amazon.com/gp/product/B08HQ1VSVL
 * 3d Printed Case: https://www.thingiverse.com/thing:3383359 
 * Picture / logo converter: https://marlinfw.org/tools/u8glib/converter.html - Marlin 2.x, ASCII Art, and Bitmap should be selected.  
 * Code written using the Arduino IDE version 1.8.13 and is a derivative of C/C++ 
 *    
 */
 
//Include the I2C library
#include <Wire.h>

//Include the Temp / Pressure / Humidity / Altitude library
#include <SparkFunBME280.h>

//Include the C02 / VoC library
#include <SparkFunCCS811.h>

//Include the 128x64 display library
#include <U8glib.h>

//Set the I2C address of the C02 / VoC sensor
#define CCS811_ADDR 0x5B //Choose between default I2C address of 0x5B or alternate I2C address of 0x5A (must short solder pad on rear of sensor)

//Define the pins used for the 128x64 LCD display
#define LCD_PINS_RS        16
#define LCD_PINS_ENABLE    17
#define LCD_PINS_D4        23

//Define the pins used for the pushbutton encoder knob
#define BTN_EN1            31 //CW rotation - for future use
#define BTN_EN2            33 //CCW rotation - for future use
#define BTN_ENC            35 //Push Button - I'm using this to switch between Imperial and Metric readouts

//Define pins used for the beeper and the kill switch (The beeper is located under the smaller 2nd hole, and the kill switch is located under the larger 1st hole)
#define BEEPER             37
#define KILL               41 // For future use 

//Setup the debounce for the pushbutton
int state = LOW;      // the current state of the output pin
int reading;           // the current reading from the input pin
int previous = LOW;    // the previous reading from the input pin

// the follow variables are long's because the time, measured in miliseconds,
// will quickly become a bigger number than can be stored in an int.
long time = 0;         // the last time the output pin was toggled
long debounce = 200;   // the debounce time, increase if the output flickers


//Global sensor objects
CCS811 myCCS811(CCS811_ADDR);
BME280 myBME280;


//Initialize the 128x64 LCD Display with the pins defined above
U8GLIB_ST7920_128X64_1X u8g(LCD_PINS_D4, LCD_PINS_ENABLE, LCD_PINS_RS);  

//Define the font for the display
const u8g_fntpgm_uint8_t u8g_font_6x9[2300] U8G_SECTION(".progmem.u8g_font_6x9") = {
  0,   6,   9,   0,   254, 6,   1,   137, 2,   254, 32,  255, 254, 7,   254, 6,   254, 0,   0,   0,   6,   0,   7,
  1,   6,   6,   6,   2,   0,   128, 128, 128, 128, 0,   128, 3,   3,   3,   6,   1,   3,   160, 160, 160, 5,   7,
  7,   6,   0,   255, 80,  80,  248, 80,  248, 80,  80,  5,   9,   9,   6,   0,   254, 32,  112, 168, 160, 112, 40,
  168, 112, 32,  6,   8,   8,   6,   0,   255, 64,  168, 72,  16,  32,  72,  84,  8,   5,   7,   7,   6,   0,   255,
  96,  144, 144, 96,  152, 144, 104, 1,   3,   3,   6,   2,   3,   128, 128, 128, 2,   7,   7,   6,   2,   255, 64,
  128, 128, 128, 128, 128, 64,  2,   7,   7,   6,   2,   255, 128, 64,  64,  64,  64,  64,  128, 5,   5,   5,   6,
  0,   0,   136, 80,  248, 80,  136, 5,   5,   5,   6,   0,   0,   32,  32,  248, 32,  32,  2,   4,   4,   6,   2,
  254, 192, 64,  64,  128, 5,   1,   1,   6,   0,   2,   248, 2,   2,   2,   6,   2,   0,   192, 192, 4,   6,   6,
  6,   1,   0,   16,  16,  32,  64,  128, 128, 4,   6,   6,   6,   1,   0,   96,  144, 144, 144, 144, 96,  3,   6,
  6,   6,   1,   0,   64,  192, 64,  64,  64,  224, 4,   6,   6,   6,   1,   0,   96,  144, 16,  32,  64,  240, 4,
  6,   6,   6,   1,   0,   240, 32,  96,  16,  16,  224, 5,   6,   6,   6,   0,   0,   16,  48,  80,  144, 248, 16,
  4,   6,   6,   6,   1,   0,   240, 128, 224, 16,  16,  224, 4,   6,   6,   6,   1,   0,   96,  128, 224, 144, 144,
  96,  4,   6,   6,   6,   1,   0,   240, 16,  16,  32,  64,  64,  4,   6,   6,   6,   1,   0,   96,  144, 96,  144,
  144, 96,  4,   6,   6,   6,   1,   0,   96,  144, 144, 112, 16,  96,  2,   5,   5,   6,   2,   0,   192, 192, 0,
  192, 192, 2,   7,   7,   6,   2,   254, 192, 192, 0,   192, 64,  64,  128, 5,   5,   5,   6,   0,   0,   24,  96,
  128, 96,  24,  5,   3,   3,   6,   0,   1,   248, 0,   248, 5,   5,   5,   6,   0,   0,   192, 48,  8,   48,  192,
  4,   7,   7,   6,   1,   0,   96,  144, 16,  96,  64,  0,   64,  5,   6,   6,   6,   0,   0,   112, 144, 168, 176,
  128, 112, 5,   6,   6,   6,   0,   0,   32,  80,  136, 248, 136, 136, 5,   6,   6,   6,   0,   0,   240, 136, 240,
  136, 136, 240, 4,   6,   6,   6,   1,   0,   96,  144, 128, 128, 144, 96,  4,   6,   6,   6,   1,   0,   224, 144,
  144, 144, 144, 224, 4,   6,   6,   6,   1,   0,   240, 128, 224, 128, 128, 240, 4,   6,   6,   6,   1,   0,   240,
  128, 224, 128, 128, 128, 4,   6,   6,   6,   1,   0,   96,  144, 128, 176, 144, 96,  4,   6,   6,   6,   1,   0,
  144, 144, 240, 144, 144, 144, 3,   6,   6,   6,   1,   0,   224, 64,  64,  64,  64,  224, 5,   6,   6,   6,   0,
  0,   56,  16,  16,  16,  144, 96,  4,   6,   6,   6,   1,   0,   144, 160, 192, 160, 144, 144, 4,   6,   6,   6,
  1,   0,   128, 128, 128, 128, 128, 240, 5,   6,   6,   6,   0,   0,   136, 216, 168, 168, 136, 136, 4,   6,   6,
  6,   1,   0,   144, 208, 176, 144, 144, 144, 5,   6,   6,   6,   0,   0,   112, 136, 136, 136, 136, 112, 4,   6,
  6,   6,   1,   0,   224, 144, 144, 224, 128, 128, 4,   7,   7,   6,   1,   255, 96,  144, 144, 208, 176, 96,  16,
  4,   6,   6,   6,   1,   0,   224, 144, 144, 224, 144, 144, 4,   6,   6,   6,   1,   0,   96,  144, 64,  32,  144,
  96,  5,   6,   6,   6,   0,   0,   248, 32,  32,  32,  32,  32,  4,   6,   6,   6,   1,   0,   144, 144, 144, 144,
  144, 96,  4,   6,   6,   6,   1,   0,   144, 144, 144, 240, 96,  96,  5,   6,   6,   6,   0,   0,   136, 136, 168,
  168, 216, 136, 5,   6,   6,   6,   0,   0,   136, 80,  32,  32,  80,  136, 5,   6,   6,   6,   0,   0,   136, 136,
  80,  32,  32,  32,  4,   6,   6,   6,   1,   0,   240, 16,  32,  64,  128, 240, 3,   6,   6,   6,   1,   0,   224,
  128, 128, 128, 128, 224, 4,   6,   6,   6,   1,   0,   128, 128, 64,  32,  16,  16,  3,   6,   6,   6,   1,   0,
  224, 32,  32,  32,  32,  224, 5,   3,   3,   6,   0,   3,   32,  80,  136, 5,   1,   1,   6,   0,   254, 248, 2,
  2,   2,   6,   2,   4,   128, 64,  4,   4,   4,   6,   1,   0,   112, 144, 144, 112, 4,   6,   6,   6,   1,   0,
  128, 128, 224, 144, 144, 224, 4,   4,   4,   6,   1,   0,   112, 128, 128, 112, 4,   6,   6,   6,   1,   0,   16,
  16,  112, 144, 144, 112, 4,   4,   4,   6,   1,   0,   96,  176, 192, 112, 4,   6,   6,   6,   1,   0,   32,  80,
  64,  224, 64,  64,  4,   6,   6,   6,   1,   254, 96,  144, 144, 112, 16,  96,  4,   6,   6,   6,   1,   0,   128,
  128, 224, 144, 144, 144, 3,   6,   6,   6,   1,   0,   64,  0,   192, 64,  64,  224, 3,   8,   8,   6,   1,   254,
  32,  0,   96,  32,  32,  32,  160, 64,  4,   6,   6,   6,   1,   0,   128, 128, 160, 192, 160, 144, 3,   6,   6,
  6,   1,   0,   192, 64,  64,  64,  64,  224, 5,   4,   4,   6,   0,   0,   208, 168, 168, 136, 4,   4,   4,   6,
  1,   0,   224, 144, 144, 144, 4,   4,   4,   6,   1,   0,   96,  144, 144, 96,  4,   6,   6,   6,   1,   254, 224,
  144, 144, 224, 128, 128, 4,   6,   6,   6,   1,   254, 112, 144, 144, 112, 16,  16,  4,   4,   4,   6,   1,   0,
  160, 208, 128, 128, 4,   4,   4,   6,   1,   0,   112, 192, 48,  224, 4,   6,   6,   6,   1,   0,   64,  64,  224,
  64,  80,  32,  4,   4,   4,   6,   1,   0,   144, 144, 144, 112, 4,   4,   4,   6,   1,   0,   144, 144, 96,  96,
  5,   4,   4,   6,   0,   0,   136, 168, 168, 80,  4,   4,   4,   6,   1,   0,   144, 96,  96,  144, 4,   6,   6,
  6,   1,   254, 144, 144, 144, 112, 144, 96,  4,   4,   4,   6,   1,   0,   240, 32,  64,  240, 3,   7,   7,   6,
  1,   0,   32,  64,  64,  128, 64,  64,  32,  1,   7,   7,   6,   2,   255, 128, 128, 128, 128, 128, 128, 128, 3,
  7,   7,   6,   1,   0,   128, 64,  64,  32,  64,  64,  128, 4,   2,   2,   6,   1,   3,   80,  160, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 0,   0,   0,   6,   0,   7,   1,   6,   6,   6,   2,   0,   128, 0,   128, 128,
  128, 128, 4,   6,   6,   6,   1,   255, 32,  112, 160, 160, 112, 32,  5,   7,   7,   6,   0,   255, 48,  72,  64,
  240, 64,  64,  248, 5,   5,   5,   6,   0,   0,   168, 80,  136, 80,  168, 5,   6,   6,   6,   0,   0,   136, 80,
  248, 32,  248, 32,  1,   7,   7,   6,   2,   255, 128, 128, 128, 0,   128, 128, 128, 4,   7,   7,   6,   1,   255,
  112, 128, 96,  144, 96,  16,  224, 3,   1,   1,   6,   1,   5,   160, 6,   7,   7,   6,   0,   0,   120, 132, 148,
  164, 148, 132, 120, 3,   5,   5,   6,   1,   1,   96,  160, 96,  0,   224, 5,   5,   5,   6,   0,   0,   40,  80,
  160, 80,  40,  4,   3,   3,   6,   1,   0,   240, 16,  16,  4,   1,   1,   6,   1,   2,   240, 6,   7,   7,   6,
  0,   0,   120, 132, 180, 164, 164, 132, 120, 4,   1,   1,   6,   1,   5,   240, 4,   3,   3,   6,   1,   2,   96,
  144, 96,  5,   7,   7,   6,   0,   255, 32,  32,  248, 32,  32,  0,   248, 3,   5,   5,   6,   1,   1,   64,  160,
  32,  64,  224, 3,   5,   5,   6,   1,   1,   192, 32,  64,  32,  192, 2,   2,   2,   6,   2,   4,   64,  128, 4,
  5,   5,   6,   1,   255, 144, 144, 176, 208, 128, 5,   6,   6,   6,   0,   0,   120, 232, 232, 104, 40,  40,  1,
  1,   1,   6,   2,   2,   128, 2,   2,   2,   6,   2,   254, 64,  128, 3,   5,   5,   6,   1,   1,   64,  192, 64,
  64,  224, 3,   5,   5,   6,   1,   1,   64,  160, 64,  0,   224, 5,   5,   5,   6,   0,   0,   160, 80,  40,  80,
  160, 5,   8,   8,   6,   0,   255, 64,  192, 64,  80,  112, 48,  120, 16,  5,   8,   8,   6,   0,   255, 64,  192,
  64,  80,  104, 8,   16,  56,  5,   8,   8,   6,   0,   255, 192, 32,  64,  48,  240, 48,  120, 16,  4,   7,   7,
  6,   1,   0,   32,  0,   32,  96,  128, 144, 96,  5,   7,   7,   6,   0,   0,   64,  32,  32,  80,  112, 136, 136,
  5,   7,   7,   6,   0,   0,   16,  32,  32,  80,  112, 136, 136, 5,   7,   7,   6,   0,   0,   32,  80,  32,  80,
  112, 136, 136, 5,   7,   7,   6,   0,   0,   40,  80,  32,  80,  112, 136, 136, 5,   7,   7,   6,   0,   0,   80,
  0,   32,  80,  112, 136, 136, 5,   7,   7,   6,   0,   0,   32,  80,  32,  80,  112, 136, 136, 5,   6,   6,   6,
  0,   0,   120, 160, 240, 160, 160, 184, 4,   8,   8,   6,   1,   254, 96,  144, 128, 128, 144, 96,  32,  64,  4,
  7,   7,   6,   1,   0,   64,  32,  240, 128, 224, 128, 240, 4,   7,   7,   6,   1,   0,   32,  64,  240, 128, 224,
  128, 240, 4,   7,   7,   6,   1,   0,   32,  80,  240, 128, 224, 128, 240, 4,   7,   7,   6,   1,   0,   80,  0,
  240, 128, 224, 128, 240, 3,   7,   7,   6,   1,   0,   128, 64,  224, 64,  64,  64,  224, 3,   7,   7,   6,   1,
  0,   32,  64,  224, 64,  64,  64,  224, 3,   7,   7,   6,   1,   0,   64,  160, 224, 64,  64,  64,  224, 3,   7,
  7,   6,   1,   0,   160, 0,   224, 64,  64,  64,  224, 5,   6,   6,   6,   0,   0,   112, 72,  232, 72,  72,  112,
  4,   7,   7,   6,   1,   0,   80,  160, 144, 208, 176, 144, 144, 4,   7,   7,   6,   1,   0,   64,  32,  96,  144,
  144, 144, 96,  4,   7,   7,   6,   1,   0,   32,  64,  96,  144, 144, 144, 96,  4,   7,   7,   6,   1,   0,   32,
  80,  96,  144, 144, 144, 96,  4,   7,   7,   6,   1,   0,   80,  160, 96,  144, 144, 144, 96,  4,   7,   7,   6,
  1,   0,   80,  0,   96,  144, 144, 144, 96,  5,   5,   5,   6,   0,   0,   136, 80,  32,  80,  136, 4,   8,   8,
  6,   1,   255, 16,  112, 176, 176, 208, 208, 224, 128, 4,   7,   7,   6,   1,   0,   64,  32,  144, 144, 144, 144,
  96,  4,   7,   7,   6,   1,   0,   32,  64,  144, 144, 144, 144, 96,  4,   7,   7,   6,   1,   0,   32,  80,  144,
  144, 144, 144, 96,  4,   7,   7,   6,   1,   0,   80,  0,   144, 144, 144, 144, 96,  5,   7,   7,   6,   0,   0,
  16,  32,  136, 80,  32,  32,  32,  4,   6,   6,   6,   1,   0,   128, 224, 144, 144, 224, 128, 4,   6,   6,   6,
  1,   0,   96,  144, 160, 160, 144, 160, 4,   7,   7,   6,   1,   0,   64,  32,  0,   112, 144, 144, 112, 4,   7,
  7,   6,   1,   0,   32,  64,  0,   112, 144, 144, 112, 4,   7,   7,   6,   1,   0,   32,  80,  0,   112, 144, 144,
  112, 4,   7,   7,   6,   1,   0,   80,  160, 0,   112, 144, 144, 112, 4,   6,   6,   6,   1,   0,   80,  0,   112,
  144, 144, 112, 4,   7,   7,   6,   1,   0,   32,  80,  32,  112, 144, 144, 112, 5,   4,   4,   6,   0,   0,   112,
  168, 176, 120, 4,   6,   6,   6,   1,   254, 112, 128, 128, 112, 32,  64,  4,   7,   7,   6,   1,   0,   64,  32,
  0,   96,  176, 192, 112, 4,   7,   7,   6,   1,   0,   32,  64,  0,   96,  176, 192, 112, 4,   7,   7,   6,   1,
  0,   32,  80,  0,   96,  176, 192, 112, 4,   6,   6,   6,   1,   0,   80,  0,   96,  176, 192, 112, 3,   7,   7,
  6,   1,   0,   128, 64,  0,   192, 64,  64,  224, 3,   7,   7,   6,   1,   0,   32,  64,  0,   192, 64,  64,  224,
  3,   7,   7,   6,   1,   0,   64,  160, 0,   192, 64,  64,  224, 3,   6,   6,   6,   1,   0,   160, 0,   192, 64,
  64,  224, 4,   7,   7,   6,   1,   0,   48,  96,  16,  112, 144, 144, 96,  4,   7,   7,   6,   1,   0,   80,  160,
  0,   224, 144, 144, 144, 4,   7,   7,   6,   1,   0,   64,  32,  0,   96,  144, 144, 96,  4,   7,   7,   6,   1,
  0,   32,  64,  0,   96,  144, 144, 96,  4,   7,   7,   6,   1,   0,   32,  80,  0,   96,  144, 144, 96,  4,   7,
  7,   6,   1,   0,   80,  160, 0,   96,  144, 144, 96,  4,   6,   6,   6,   1,   0,   80,  0,   96,  144, 144, 96,
  5,   5,   5,   6,   0,   0,   32,  0,   248, 0,   32,  4,   4,   4,   6,   1,   0,   112, 176, 208, 224, 4,   7,
  7,   6,   1,   0,   64,  32,  0,   144, 144, 144, 112, 4,   7,   7,   6,   1,   0,   32,  64,  0,   144, 144, 144,
  112, 4,   7,   7,   6,   1,   0,   32,  80,  0,   144, 144, 144, 112, 4,   6,   6,   6,   1,   0,   80,  0,   144,
  144, 144, 112, 4,   9,   9,   6,   1,   254, 32,  64,  0,   144, 144, 144, 112, 144, 96,  4,   8,   8,   6,   1,
  254, 128, 128, 224, 144, 144, 224, 128, 128, 4,   8,   8,   6,   1,   254, 80,  0,   144, 144, 144, 112, 144, 96
};

//Define the logo image for the splash screen - https://marlinfw.org/tools/u8glib/converter.html - Marlin 2.x, ASCII Art, and Bitmap should be selected.
const unsigned char TownLogoImage[] PROGMEM = {
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ................................................................................................................................
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ................................................................................................................................
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ................................................................................................................................
  0x1F,0xFF,0xFF,0xF8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ...##########################...................................................................................................
  0x1F,0xFF,0xFF,0xF8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ...##########################...................................................................................................
  0x1F,0xFF,0xFF,0xF8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ...##########################...................................................................................................
  0x1E,0x07,0xC0,0xF8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ...####......#####......#####...................................................................................................
  0x3C,0x07,0xC0,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ..####.......#####.......####...................................................................................................
  0x3C,0x07,0xC0,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ..####.......#####........###...................................................................................................
  0x3C,0x07,0xC0,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ..####.......#####........###...................................................................................................
  0x38,0x07,0xC0,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ..###........#####.........##...................................................................................................
  0x00,0x07,0xC0,0x00,0x1C,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x1C,0x00,0x00,0x00, // .............#####.................###.....................................................#.......###..........................
  0x00,0x07,0xC0,0x01,0xFF,0xC7,0xFF,0xCF,0xFF,0x9F,0xFB,0xF0,0xFF,0x00,0x00,0x00, // .............#####.............###########...#############..#############..##########.######....########........................
  0x00,0x07,0xC0,0x03,0xFF,0xE3,0xFF,0xCF,0xFF,0x9F,0xFF,0xF1,0xFF,0x80,0x00,0x00, // .............#####............#############...############..#############..#################...##########.......................
  0x00,0x07,0xC0,0x07,0xC1,0xF9,0xFE,0x03,0xFC,0x07,0xFF,0xF7,0xFF,0xC0,0x00,0x00, // .............#####...........#####.....######..########.......########.......###############.#############......................
  0x00,0x07,0xC0,0x1F,0x00,0xF8,0xFE,0x01,0xF8,0x01,0xCF,0xFF,0x87,0xC0,0x00,0x00, // .............#####.........#####........#####...#######........######..........###..#############....#####......................
  0x00,0x07,0xC0,0x1F,0x00,0x7C,0x7E,0x00,0xF8,0x01,0x83,0xFE,0x03,0xE0,0x00,0x00, // .............#####.........#####.........#####...######.........#####..........##.....#########.......#####.....................
  0x00,0x07,0xC0,0x3E,0x00,0x7E,0x7E,0x00,0xF8,0x03,0x81,0xF8,0x03,0xE0,0x00,0x00, // .............#####........#####..........######..######.........#####.........###......######.........#####.....................
  0x00,0x07,0xC0,0x3C,0x00,0x3E,0x3E,0x00,0xFC,0x03,0x81,0xF0,0x03,0xE0,0x00,0x00, // .............#####........####............#####...#####.........######........###......#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x3E,0x3F,0x01,0xFC,0x07,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####............#####...######.......#######.......###.......#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1F,0x3F,0x01,0xFE,0x07,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............#####..######.......########......###.......#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1F,0x1F,0x01,0xFE,0x07,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............#####...#####.......########......###.......#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1F,0x1F,0x83,0xBE,0x0E,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............#####...######.....###.#####.....###........#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1F,0x0F,0x83,0xBE,0x0E,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............#####....#####.....###.#####.....###........#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1F,0x0F,0xC7,0x1F,0x0C,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............#####....######...###...#####....##.........#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1F,0x0F,0xC7,0x1F,0x1C,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............#####....######...###...#####...###.........#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1F,0x07,0xC6,0x0F,0x98,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............#####.....#####...##.....#####..##..........#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x1E,0x07,0xCE,0x0F,0xB8,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####.............####......#####..###.....#####.###..........#####..........#####.....................
  0x00,0x07,0xC0,0x7C,0x00,0x3E,0x07,0xFE,0x0F,0xF8,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####.......#####............#####......##########.....#########..........#####..........#####.....................
  0x00,0x07,0xC0,0x3C,0x00,0x3E,0x03,0xFC,0x0F,0xF0,0x01,0xF0,0x03,0xE0,0x00,0x00, // .............#####........####............#####.......########......########...........#####..........#####.....................
  0x00,0x07,0xC0,0x3E,0x00,0x7C,0x01,0xFC,0x07,0xF0,0x01,0xF0,0x03,0xE0,0xF8,0x00, // .............#####........#####..........#####.........#######.......#######...........#####..........#####.....#####...........
  0x00,0x07,0xC0,0x1F,0x00,0xF8,0x01,0xF8,0x07,0xF0,0x01,0xF0,0x03,0xE0,0xFC,0x00, // .............#####.........#####........#####..........######........#######...........#####..........#####.....######..........
  0x00,0x0F,0xE0,0x0F,0xC1,0xF8,0x01,0xF8,0x07,0xE0,0x03,0xF8,0x07,0xE0,0xFC,0x00, // ............#######.........######.....######..........######........######...........#######........######.....######..........
  0x00,0x3F,0xF8,0x07,0xFF,0xF0,0x01,0xF8,0x03,0xE0,0x0F,0xFE,0x1F,0xFC,0xFC,0x00, // ..........###########........###############...........######.........#####.........###########....###########..######..........
  0x00,0x7F,0xFC,0x03,0xFF,0xC0,0x00,0xF0,0x03,0xC0,0x1F,0xFF,0x1F,0xFE,0xFC,0x00, // .........#############........############..............####..........####.........#############...############.######..........
  0x00,0x00,0x00,0x00,0x3E,0x00,0x00,0xC0,0x01,0x80,0x00,0x00,0x00,0x00,0x30,0x00, // ..................................#####.................##.............##.........................................##............
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ................................................................................................................................
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // ................................................................................................................................
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00  // ................................................................................................................................
};

//Show intro splash screen - Called from setup()
void splash(void) {

  //Draw a box around the outer edge
  u8g.drawFrame(0, 0, 128, 64);
  
  u8g.drawBitmapP(3, 0, 16, 39, TownLogoImage);

  u8g.setFont(u8g_font_blipfest_07);
        
  u8g.setPrintPos(4, 44); 
  u8g.print("Mrs. Vereen's Environmental Sensor");
  u8g.setPrintPos(4, 52); 
  u8g.print("                                          Version 1.0");
  u8g.setPrintPos(4, 60); 
  u8g.print("              Made with love by Kevin Cook");
}

//Reads sensor data and display it on the screen -  Called from Loop()
void draw(void) {
  //Set the font to use  
  u8g.setFont(u8g_font_6x9);
  //u8g.setFont(u8g_font_helvR08);
  //u8g.setFont(u8g_font_04b_03);
  //u8g.setFont(u8g_font_5x7);
  //u8g.setFont(u8g_font_5x8);
  //u8g.setFont(u8g_font_profont12);
  //u8g.setFont(u8g_font_profont11);
  //u8g.setFont(u8g_font_trixel_square);
  //u8g.setFont(u8g_font_chikita);
  
  //Draw a box around the outer edge
  u8g.drawFrame(0, 0, 128, 64);

  //Get CCS811 CO2 / VOC sensor results
  myCCS811.readAlgorithmResults();

  //Read BME280 Temp / Humidity sensor settings and pass them to the CCS811 CO2 / VOC sensor for more accurate readings
  float BMEtempC = myBME280.readTempC();
  float BMEhumid = myBME280.readFloatHumidity();
  
  //This sends the temperature data to the CCS811
  myCCS811.setEnvironmentalData(BMEhumid, BMEtempC);
    
  //Read CO2 level and display it
  u8g.setPrintPos(2, 8); 
  u8g.print("CO2 Level: ");
  u8g.print(myCCS811.getCO2());
  u8g.print("ppm");
  
  //Read VOC level and display it
  u8g.setPrintPos(2, 17);
  u8g.print("Total VOC: ");
  u8g.print(myCCS811.getTVOC());
  u8g.print("ppm");

  
  //Display BME280 sensor results
  
  
  //Read Altitude and display it  
  u8g.setPrintPos(2, 26);
  u8g.print("Temperature: ");
  if (state == HIGH){   
    u8g.print(myBME280.readTempC(), 1);
    u8g.print("C");
  }
  else {
  u8g.print(myBME280.readTempF(), 1);
  u8g.print("F");
  }
  
  //Read Air Pressure and display it
  u8g.setPrintPos(2, 35);
  u8g.print("Pressure: ");
  if (state == HIGH){  
    u8g.print(myBME280.readFloatPressure() / 1000, 1);
    u8g.print("kPa");
  }
  else {
    u8g.print((myBME280.readFloatPressure() * 0.0002953), 1);
    u8g.print("InHg");
  }

  //Read Humidity level and display it    
  u8g.setPrintPos(2, 44);
  u8g.print("Humidity: ");
  u8g.print(myBME280.readFloatHumidity(), 1);
  u8g.print("%");

  //Read Dew Point and display it
  u8g.setPrintPos(2, 53);
  u8g.print("Dew Point: ");
  if (state == HIGH){ 
    u8g.print(myBME280.dewPointC(), 1);
    u8g.print("C");
  }
  else {    
  u8g.print(myBME280.dewPointF(), 1);
  u8g.print("F");  
  }

  //Read Temperature and display it  
  u8g.setPrintPos(2, 62);
  u8g.print("Altitude: ");
  if (state == HIGH){
    u8g.print(myBME280.readFloatAltitudeMeters(), 1);
    u8g.print("m");
  }
  else {
    u8g.print(myBME280.readFloatAltitudeFeet(), 1);
    u8g.print("ft");
  }

}


// Runs once at startup
void setup(void) {
  //initialize I2C bus
  Wire.begin();

  //Initialize the CCS811 sensor
  myCCS811.begin();
  
  //Initialize the BME280 Temp / Pressure / Humidity / Altitude sensor
  myBME280.settings.commInterface = I2C_MODE;// Choose between I2C or SPI
  myBME280.settings.I2CAddress = 0x77;//Choose between default I2C address of 0x77 or alternate I2C address of 0x76 (must short solder pad on rear of sensor)
  myBME280.settings.runMode = 1; //3     0=Sleep, 1=Forced, 3=Normal
  myBME280.settings.tStandby = 5; //0     Time in ms - 0=.5, 1=62.5, 2=125, 3=250, 4=500, 5=1000, 6=10, 7=20
  myBME280.settings.filter = 0; //4     0=Filter off, 1=coefficient of 2, 2=coefficient of 4, 3=coefficient of 8, 4=coefficient of 16
  myBME280.settings.tempOverSample = 1; //5     0=Turns off temperature sensing, 1=oversampling *1, 2=oversampling *2, 4=oversampling *4, 8=oversampling *8, 16=oversampling *16, Other: Bad Entry, sets to oversampling *1
  myBME280.settings.pressOverSample = 1; //5     0=Turns off pressure sensing, 1=oversampling *1, 2=oversampling *2, 4=oversampling *4, 8=oversampling *8, 16=oversampling *16, Other: Bad Entry, sets to oversampling *1
  myBME280.settings.humidOverSample = 1; //5     0=Turns off humidity sensing, 1=oversampling *1, 2=oversampling *2, 4=oversampling *4, 8=oversampling *8, 16=oversampling *16, Other: Bad Entry, sets to oversampling *1
  
  //Make sure sensor had enough time to turn on. BME280 requires 2ms to start up.
  delay(10);
  
  //Calling .begin() causes the settings to be loaded
  myBME280.begin();
  
  //Set the knob pin as an input, and the beeper pin as an output
  pinMode(BTN_ENC, INPUT_PULLUP);
  pinMode(BEEPER,OUTPUT);

  //Beep the beeper
  digitalWrite(BEEPER,HIGH);
  delay(100);
  digitalWrite(BEEPER,LOW);

  //Show the splash screen
  u8g.firstPage();  
  do {
    splash();
  } while( u8g.nextPage() );

  //Redraw the picture after a delay (Since this in setup, it will just delay once and then end)
  delay(10000);
}


//Main loop - Runs continuosly after setup()
void loop(void) {
  reading = digitalRead(BTN_ENC);
  // if the input just went from LOW and HIGH and we've waited long enough
  // to ignore any noise on the circuit, toggle the output pin and remember the time
  if (reading == HIGH && previous == LOW && millis() - time > debounce) {
    if (state == HIGH)
      state = LOW;
    else
      state = HIGH;
    time = millis();    
  }

  //digitalWrite(BEEPER, state);

  previous = reading;

  // picture loop
  u8g.firstPage();  
  do {
    draw();
  } while( u8g.nextPage() );

  //Redraw the picture after a delay
  delay(500);
  
}